% Code to simulate data from a Factor copula  model and then estimate
% the parameter using SMM with correctly specified or misspecified model in Simulation Study 5.1.
%
%  Fang Duan
%
%  Last Update on Jan, 2018
%
% This code was developed for the following paper:
%
% F. Duan, H. Manner and D. Wied, 2019, "Model and Moment Selection in Factor Copula Models".
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear
clc

% cd and add SMM copula toolbox and MFE toolbox or futher help functions
addmypath;

R  = 200; % number of replications
Time = [500 1000 2000]; % different sample length
for kk = 1:3
    T = Time(kk);
    N = 3; % dimension, other possibilities: N=5, N=10
    S = 25;
    simfunc_true = 'simfunc_gaussian'; % true DGP: 'simfunc_tt', 'simfunc_skewtt', 'simfunc_skewnn', 'simfunc_gaussian'
    simfunc_est = 'simfunc_gaussian'; % factor copula used in SMM: 'simfunc_tt', 'simfunc_skewtt', 'simfunc_skewnn', 'simfunc_gaussian'
    
    if strcmp(simfunc_true,simfunc_est)
        type = 'Size_niid_'; % estimate with correctly specified model
    else
        type = 'Power_niid_'; % estimate with misspecified model
    end

    setting; % quantile dependence measure used in SMM 

    % dynamic parameters in marginals - AR(1)-GARCH(1,1)
    phi= [0.01, 0.05, 0.05, 0.10, 0.85]; % [\phi_0,\phi_1,\omega,\alpha,\beta]

    % moment function
    momfunc = 'momfunc_Equidep'; % momfunc = 'momfunc_Equidep_lower'; % for the case that rho_S is excluded
    
    %copula paramters in true DGP
    if strcmp(simfunc_true,'simfunc_gaussian')
        theta_null = 1;
    elseif strcmp(simfunc_true,'simfunc_tn') || strcmp(simfunc_true,'simfunc_tt')
        theta_null = [1 0.25];
    elseif strcmp(simfunc_true, 'simfunc_skewtt') || strcmp(simfunc_true,'simfunc_skewtn')
        theta_null = [1 0.25 -0.5];
    elseif strcmp(simfunc_true,'simfunc_skewnn')
        theta_null = [1 -0.5];
    end

    % estimation setting in SMM
    if strcmp(simfunc_est,'simfunc_gaussian')
        thetahat = zeros(R,1);
        theta_guess = 2;
        LB = 0;
        UB = 5;
    elseif strcmp(simfunc_est,'simfunc_tn') || strcmp(simfunc_est,'simfunc_tt')
        thetahat = zeros(R,2);
        theta_guess = [2 0.25];
        LB = [0 0.01];
        UB = [5 0.49];
    elseif strcmp(simfunc_est,'simfunc_skewtt') || strcmp(simfunc_est,'simfunc_skewtn')
        thetahat = zeros(R,3);
        theta_guess = [2 0.25 0];
        LB = [0 0.01 -0.99];
        UB = [5 0.49 0.99];
    elseif strcmp(simfunc_est,'simfunc_skewnn')
        thetahat = zeros(R,2);
        theta_guess = [2 0];
        LB = [0 -0.99];
        UB = [5 0.99];
    end

    % Monte Carlo Simulation
    Jstat = zeros(R,1);
    param_dynamic = zeros(R*N,length(phi));

    for ii = 1:R    
        disp(strcat('Round:',num2str(ii)))
        seed = ii;
        other_null = [T N seed];
        other2 = [S*T N 2*seed];
        options = optimset('MaxIter',1000);

        % 1&2. Draw random numbers from a given copula and corresponding return series
            [data,uresid] = simulate_return(simfunc_true,theta_null,other_null,dpd,phi);

        % 3. Estimate AR(1)-GARCH(1,1) with NORMAL innovations
            data_resid=NaN(T,N); % AR-GARCH residuals
            data_param=NaN(N,length(phi)); % AR-GARCH parameters
            data_uresid_semipar=NaN(T,N); % ARMA-GARCH PIT residuals (semiparametric)
            startval = phi;
            for nn=1:N
                [~,temp_param,~,~,temp_resid,~]=fitARMAGARCH(data(:,nn),[1,0,1,0,1],'NORMAL',startval);
                data_resid(:,nn)=temp_resid;
                data_param(nn,:)=temp_param;
                data_uresid_semipar(:,nn)=tiedrank(temp_resid)/(T+1);
            end
            % store the estimates of dynamic parameters
            param_dynamic(((ii-1)*N+1):(ii*N),:) = data_param;

        % 4. fit selected copula to the data   
        if strcmp(momfunc,'momfunc_Equidep_lower')
            W = eye(length(q));
        else
            W = eye(length(q)+1);
        end

        [thetahat(ii,:),~] = fminsearchbnd('objfunc_SMM',theta_guess,LB,UB,options,simfunc_est,other2,dpd,momfunc,data_uresid_semipar,W,q);
        [~,ghat] = objfunc_SMM(thetahat(ii,:),simfunc_est,other2,dpd,momfunc,data_uresid_semipar,W,q);
        
        % 6. J test
        Jstat(ii,1) = min(T,S*T) * ghat' * W * ghat; 
    end
    % Save the output
    disp(strcat('Simulation_niid_',simfunc_true,'_est_by_',simfunc_est,'_T',num2str(T),'_N',num2str(N),'_R',num2str(R),'_',dpd))
    save( strcat(type,simfunc_true,'_est_by_',simfunc_est,'_T',num2str(T),'_N',num2str(N),'_R',num2str(R),'_',dpd,'.mat') );
end